# Copyright (C) 2025 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from perfetto.protos.perfetto.trace.perfetto_trace_pb2 import Trace
from perfetto.protos.perfetto.trace.perfetto_trace_pb2 import TracePacket


class TraceProtoBuilder:
  """A builder for creating Perfetto traces from Python."""

  def __init__(self):
    """Initializes the TraceProtoBuilder."""
    self.trace = Trace()

  def add_packet(self) -> TracePacket:
    """Adds a packet to the trace and returns the packet for modification."""
    return self.trace.packet.add()

  def serialize(self) -> bytes:
    return self.trace.SerializeToString()
